
! Module: srcmatch_def
!
! Authors:  Jacco Vink - AIP
!          
! Description: Definitions to be used in eident_mod & eident_io
! 
! Comments: 
! 
! $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
module srcmatch_def

  use param  
  use dal

  implicit none

  integer(kind=int32), parameter :: max_nsets = 9
  integer(kind=int32), parameter :: null_entry= 0
  integer(kind=int32), parameter :: instr_all = 0
  integer(kind=int32), parameter :: instr_pn = 1, instr_m1 = 2,instr_m2 = 3  
  integer(kind=int32), parameter :: summary_band = 0, xid_band=9
  real(kind=double), parameter :: pi = 3.1415926535897932385d0
  real(kind=double), parameter :: rad2deg = 180.0d0/pi
  real(kind=double), parameter :: deg2rad = pi/180.0d0
  real(kind=double), parameter :: rad2arcsec = 3600.0d0*180.0d0/pi
  real(kind=double), parameter :: arcsec2rad = pi/(3600.0d0*180.0d0)
  real(kind=double), parameter :: deg2arcsec = 3600.0d0
  real(kind=double), parameter :: sterrad2arcsec = 4.254517029d10
  real(kind=double), parameter :: ra_gal_pole = 192.25
  real(kind=double), parameter :: dec_gal_pole = 27.4
!  real(kind=double), parameter :: ra_gal_pole =  192.8594
!  real(kind=double), parameter :: ra_gal_pole =  192.864
!  real(kind=double), parameter :: ra_gal_pole = 192.859481
!  real(kind=double), parameter :: dec_gal_pole = 27.128251
  real(kind=double), parameter :: gal_theta=123.0d0

  !
  ! coordinate type definition, corresponds to XMM-SOC-ICD-0006-SSC,(p.43,v1.1)
  ! x,y, etc. corresponds to x_ima, y_ima etc.
  !
  type :: Source_type
     integer(kind=int32) :: id_src, id_inst, id_band
     integer(kind=int32) :: src_num, dummy_id
     integer(kind=int32),dimension(max_nsets) :: ep_ml_id_src,  pn_ml_id_src
     integer(kind=int32),dimension(max_nsets) :: m1_ml_id_src, m2_ml_id_src
     integer(kind=int32),dimension(max_nsets) :: ep_box_id_src, pn_box_id_src
     integer(kind=int32),dimension(max_nsets) :: m1_box_id_src, m2_box_id_src
     real(kind=double)   :: ra, dec, ra_corr, dec_corr
     real(kind=double)   :: lii, bii, lii_corr,bii_corr
     real(kind=single)  :: radec_err
     real(kind=single)  :: pn_tot, pn_tot_err
     real(kind=single)  :: m1_tot, m1_tot_err, m2_tot, m2_tot_err
     real(kind=single)  :: ep_tot, ep_tot_err
     real(kind=single)  :: pn_xid, pn_xid_err
     real(kind=single)  :: m1_xid, m1_xid_err, m2_xid, m2_xid_err
     real(kind=single)  :: ep_xid, ep_xid_err
     real(kind=single),dimension(max_nsets) :: pn_det_ml, m1_det_ml, m2_det_ml
     real(kind=single),dimension(max_nsets) :: ep_det_ml
     real(kind=single)  :: pn_extent, pn_ext_err
     real(kind=single)  :: m1_extent, m1_ext_err
     real(kind=single)  :: m2_extent, m2_ext_err
     real(kind=single)  :: ep_extent, ep_ext_err
     real(kind=single),dimension(max_nsets) :: pn_ext_ml, m1_ext_ml, m2_ext_ml
     real(kind=single),dimension(max_nsets) :: ep_ext_ml

     real(kind=single)  :: pn_hr1, pn_hr1_err, pn_hr2, pn_hr2_err
     real(kind=single)  :: pn_hr3, pn_hr3_err
     real(kind=single)  :: m1_hr1, m1_hr1_err, m1_hr2, m1_hr2_err
     real(kind=single)  :: m1_hr3, m1_hr3_err
     real(kind=single)  :: m2_hr1, m2_hr1_err, m2_hr2, m2_hr2_err
     real(kind=single)  :: m2_hr3, m2_hr3_err
     real(kind=single)  :: ep_hr1, ep_hr1_err, ep_hr2, ep_hr2_err
     real(kind=single)  :: ep_hr3, ep_hr3_err
     real(kind=single)  :: var_stat, p_var
     integer(kind=int32) :: n_bins, var_exp_no
     integer(kind=int32),dimension(5) :: om_id
     real(kind=single),dimension(5)   :: om_id_dist
     integer(kind=int32):: n_opt_co
     character(10)      :: omflag
     logical            :: spectra, tseries
     character(12)      :: val_flag, ver_flag
     character(40)      :: ver_comm
     integer(kind=int32):: cross_ref
     real(kind=single)  :: dist
  end type Source_type

  type :: Header_type
     character(200),dimension(max_nsets) :: inputsets, crossidsets
     character(72)       :: telescop, instrume, filter, detnam, datamode
     character(72)       :: obs_mode,obs_id, exp_id, content, origin, object
     character(72)       :: date_obs, date_end
     character(72)       :: radecsys, observer
     logical             :: omcross
     integer(kind=int32),dimension(max_nsets) :: id_inst
     integer(kind=int32) :: nsets, ncrossid
     real(kind=single)   :: exposure, equinox
     real(kind=double)   :: ra_pnt, dec_pnt, pa_pnt
     real(kind=double)   :: max_err, syst_err, om_dist
     real(kind=single)   :: raoffset, deoffset, raofferr, deofferr
     real(kind=single)   :: rot_corr, rot_err, lik_hood, nmatches
     real(kind=single), dimension(3,9) :: eblo, ebhi  
  end type Header_type

  type :: Sheader_type
     integer(kind=int32)  :: dummy
  end type Sheader_type



end module srcmatch_def



