!+
! Module: srcmatch_io
!
! Authors:  Jacco Vink - AIP
!           Dieter Meinert - AIP (D.M.)
!          
! Description: I/O routines to be used in srcmatch_mod
! 
! Comments: 
! 
! $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
module srcmatch_io

  use param  
  use dal
  use srcmatch_def

  implicit none

  private

  public read_set, read_Header, read_om_set, write_summary_set, &
       write_summary_header, write_primary_header,write_html_list, init_list


contains


  !--------------------------------------------------------------------
  function read_set( filename, n_sources, has_ra_corr, Header ) &
       result( list_ptr)
  !--------------------------------------------------------------------

    type(Source_type), dimension(:), pointer :: list_ptr
    character(*), intent(in) :: filename
    integer(kind=int32) , intent (out) :: n_sources
    logical , intent (out)             :: has_ra_corr
    type(Header_type) , intent(out) :: Header

    logical :: single_instr = .TRUE.
    logical :: single_band  = .TRUE.
    integer(kind=int32) :: i, j, nrows, colnr
    integer(kind=int32),dimension(:),pointer :: src_num,id_src_ml,id_src_box
    integer(kind=int32),dimension(:),pointer :: instrument, id_band
    real(kind=double),dimension(:),pointer :: ra, dec, ra_corr, dec_corr
    real(kind=single),dimension(:),pointer :: radec_err
    real(kind=single),dimension(:),pointer :: rate, rate_err, det_ml
    real(kind=single),dimension(:),pointer :: ext, ext_err, ext_ml
    real(kind=single),dimension(:),pointer :: hr1, hr1_err, hr2, hr2_err
    real(kind=single),dimension(:),pointer :: hr3, hr3_err

    type(DataSetT) :: set
    type(TableT)  :: source_tab
    type(ColumnT),dimension(100) :: col

    set = dataSet(filename,READ)
    call read_Header(set, Header )

    source_tab = table( set,'SRCLIST')

    ! get number of rows in this table
    nrows = numberOfRows(source_tab)
    !3.7.3 if( nrows <= 0 ) then
    !        call release(source_tab) ! probably superfluous
    !        call release(set)        ! probably superfluous
    !        call error('emptyInputSet','EMLDETECT source list is empty')
    !3.7.3 end if

    colnr=0

    colnr = colnr + 1
    col(colnr) = column(source_tab, "ML_ID_SRC",READ)
    src_num => int32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab, "ML_ID_SRC",READ)
    id_src_ml => int32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab, "BOX_ID_SRC",READ)
    id_src_box => int32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab, "ID_INST",READ)
    instrument => int32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab, "ID_BAND",READ)
    id_band => int32Data( col(colnr) )

    colnr = colnr + 1
    col(colnr) = column(source_tab, "RA",READ)
    ra => real64Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"DEC", READ)
    dec => real64Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"RADEC_ERR", READ)
    radec_err => real32Data( col(colnr) )

    colnr = colnr + 1
    col(colnr) = column(source_tab,"RATE", READ)
    rate => real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"RATE_ERR", READ)
    rate_err => real32Data( col(colnr))
    colnr = colnr + 1
    col(colnr) = column(source_tab,"DET_ML", READ)
    det_ml =>real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"HR1", READ)
    hr1 => real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"HR1_ERR", READ)
    hr1_err => real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"HR2", READ)
    hr2 => real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"HR2_ERR", READ)
    hr2_err => real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"HR3", READ)
    hr3 => real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"HR3_ERR", READ)
    hr3_err => real32Data( col(colnr) )

    colnr = colnr + 1
    col(colnr) = column(source_tab,"EXT", READ)
    ext =>real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"EXT_ERR", READ)
    ext_err =>real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"EXT_ML", READ)
    ext_ml =>real32Data( col(colnr) )



    has_ra_corr = .FALSE.
    if(  hasAttribute(source_tab,"LIK_HOOD") ) then !eposcorr has been applied
       Header%lik_hood = real32Attribute(source_tab,"LIK_HOOD")
       Header%raoffset = real32Attribute(source_tab,"RAOFFSET")
       Header%deoffset = real32Attribute(source_tab,"DEOFFSET")
       Header%raofferr = real32Attribute(source_tab,"RAOFFERR")
       Header%deofferr = real32Attribute(source_tab,"DEOFFERR")
       Header%rot_corr = real32Attribute(source_tab,"ROT_CORR")
       Header%rot_err  = real32Attribute(source_tab,"ROT_ERR")
       Header%nmatches = real32Attribute(source_tab,"NMATCHES")
       colnr = colnr + 1
       col(colnr) = column(source_tab, "RA_CORR",READ)
       ra_corr => real64Data( col(colnr) )
       colnr = colnr + 1
       col(colnr) = column(source_tab,"DEC_CORR", READ)
       dec_corr => real64Data( col(colnr) )
       has_ra_corr = .TRUE.
    end if

! Due to the complicated input list first a number of checks need to
! be done.
! The first one is to check whether there exist rows with 
! instrument(i-1) == instr_all. If not it is a single instrument source list.
!
    single_instr = .TRUE.
    do i=1, nrows
       if( instrument(i-1) == instr_all ) then
          single_instr = .FALSE.
          exit
       end if
    end do

    if (single_instr .and. nrows>0) then   ! 3.7.3
      Header%id_inst(1) = instrument(1)
    else
      Header%id_inst(1) = 0
    end if 

! idem but now for the energy band, check whether summary_band info exists
    single_band = .TRUE.
    do i=1, nrows
       if( id_band(i-1) == summary_band ) then
          single_band = .FALSE.
          exit
       end if
    end do

    ! allocate position list
    allocate(list_ptr(nrows))
    call init_list(list_ptr, nrows)
    n_sources =0 
    do i=1, nrows
       if( (instrument(i-1) == instr_all .OR. single_instr) .AND. &
            (id_band(i-1) == summary_band .OR. single_band) ) then
          n_sources = n_sources + 1
          list_ptr(i)%dummy_id = n_sources
          list_ptr(i)%src_num  = src_num(i-1)
       else
          list_ptr(i)%dummy_id = 0
       end if
    end do

!    print "(A I4)", 'nsrcs= ',   n_sources 

! associate all row entries with a source number:
!
    do i=1, nrows
       if(list_ptr(i)%dummy_id == 0 ) then
          do j=1,nrows
             if( src_num(i-1) == list_ptr(j)%src_num ) then
                list_ptr(i)%dummy_id = list_ptr(j)%dummy_id
             end if
          end do
       end if
    end do

    ! Copy the fits data into the structure
    !
    n_sources = 0
    do i=1, nrows
       ! the -1 is the diff. between F90 and C(++)
       j = list_ptr(i)%dummy_id
       if( (instrument(i-1) == instr_all .OR. single_instr) .AND. &
            (id_band(i-1) == summary_band .OR. single_band) ) then
          n_sources = n_sources + 1
          
          list_ptr(j)%cross_ref = 0 ! initialize
          list_ptr(j)%src_num = src_num(i-1) 
          list_ptr(j)%ra = ra(i-1) 
          list_ptr(j)%dec = dec(i-1)
          list_ptr(j)%radec_err = radec_err(i-1)
          if( has_ra_corr ) then 
             list_ptr(j)%ra_corr = ra_corr(i-1) 
             list_ptr(j)%dec_corr = dec_corr(i-1)
          end if
       end if
       
       if( id_band(i-1) == summary_band ) then
          if( instrument(i-1) == instr_all .OR. single_instr ) then
             list_ptr(j)%ep_ml_id_src(1) =  id_src_ml(i-1)
             list_ptr(j)%ep_box_id_src(1) =  id_src_box(i-1)
             list_ptr(j)%ep_tot     = rate(i-1) 
             list_ptr(j)%ep_tot_err = rate_err(i-1) 
             list_ptr(j)%ep_det_ml(1)  = det_ml(i-1)
             list_ptr(j)%ep_extent  = ext(i-1)
             list_ptr(j)%ep_ext_err = ext_err(i-1)
             list_ptr(j)%ep_ext_ml(1) = ext_ml(i-1)
             list_ptr(j)%ep_hr1     = hr1(i-1)
             list_ptr(j)%ep_hr1_err = hr1_err(i-1)
             list_ptr(j)%ep_hr2     = hr2(i-1)
             list_ptr(j)%ep_hr2_err = hr2_err(i-1)
             list_ptr(j)%ep_hr3     = hr3(i-1)
             list_ptr(j)%ep_hr3_err = hr3_err(i-1)
          end if
          if( instrument(i-1) == instr_pn) then
             list_ptr(j)%pn_ml_id_src(1) =  id_src_ml(i-1)
             list_ptr(j)%pn_box_id_src(1) =  id_src_box(i-1)
             list_ptr(j)%pn_tot     = rate(i-1) 
             list_ptr(j)%pn_tot_err = rate_err(i-1)
             list_ptr(j)%pn_det_ml(1)  = det_ml(i-1)
             list_ptr(j)%pn_extent  = ext(i-1)
             list_ptr(j)%pn_ext_err = ext_err(i-1)
             list_ptr(j)%pn_ext_ml(1)  = ext_ml(i-1)
             list_ptr(j)%pn_hr1     = hr1(i-1)
             list_ptr(j)%pn_hr1_err = hr1_err(i-1)
             list_ptr(j)%pn_hr2     = hr2(i-1)
             list_ptr(j)%pn_hr2_err = hr2_err(i-1)
             list_ptr(j)%pn_hr3     = hr3(i-1)
             list_ptr(j)%pn_hr3_err = hr3_err(i-1)
          end if
          if( instrument(i-1) == instr_m1) then
             list_ptr(j)%m1_ml_id_src(1) =  id_src_ml(i-1)
             list_ptr(j)%m1_box_id_src(1) =  id_src_box(i-1)
             list_ptr(j)%m1_tot     = rate(i-1) 
             list_ptr(j)%m1_tot_err = rate_err(i-1) 
             list_ptr(j)%m1_det_ml(1) = det_ml(i-1)
             list_ptr(j)%m1_extent  = ext(i-1)
             list_ptr(j)%m1_ext_err = ext_err(i-1)
             list_ptr(j)%m1_ext_ml(1) = ext_ml(i-1)
             list_ptr(j)%m1_hr1     = hr1(i-1)
             list_ptr(j)%m1_hr1_err = hr1_err(i-1)
             list_ptr(j)%m1_hr2     = hr2(i-1)
             list_ptr(j)%m1_hr2_err = hr2_err(i-1)
             list_ptr(j)%m1_hr3     = hr3(i-1)
             list_ptr(j)%m1_hr3_err = hr3_err(i-1)
          end if
          if( instrument(i-1) == instr_m2) then
             list_ptr(j)%m2_ml_id_src(1) =  id_src_ml(i-1)
             list_ptr(j)%m2_box_id_src(1) =  id_src_box(i-1)
             list_ptr(j)%m2_tot     = rate(i-1) 
             list_ptr(j)%m2_tot_err = rate_err(i-1) 
             list_ptr(j)%m2_det_ml(1)  = det_ml(i-1)
             list_ptr(j)%m2_extent  = ext(i-1)
             list_ptr(j)%m2_ext_err = ext_err(i-1)
             list_ptr(j)%m2_ext_ml(1)  = ext_ml(i-1)
             list_ptr(j)%m2_hr1     = hr1(i-1)
             list_ptr(j)%m2_hr1_err = hr1_err(i-1)
             list_ptr(j)%m2_hr2     = hr2(i-1)
             list_ptr(j)%m2_hr2_err = hr2_err(i-1)
             list_ptr(j)%m2_hr3     = hr3(i-1)
             list_ptr(j)%m2_hr3_err = hr3_err(i-1)
          end if
       end if
       if( id_band(i-1) == xid_band) then
          if( instrument(i-1) == instr_all) then
             list_ptr(j)%ep_xid     = rate(i-1) 
             list_ptr(j)%ep_xid_err = rate_err(i-1)
          end if
          if( instrument(i-1) == instr_pn) then
             list_ptr(j)%pn_xid     = rate(i-1) 
             list_ptr(j)%pn_xid_err = rate_err(i-1)
          end if
          if( instrument(i-1) == instr_m1) then
             list_ptr(j)%m1_xid     = rate(i-1) 
             list_ptr(j)%m1_xid_err = rate_err(i-1) 
          end if
          if( instrument(i-1) == instr_m2) then
             list_ptr(j)%m2_xid     = rate(i-1) 
             list_ptr(j)%m2_xid_err = rate_err(i-1) 
          end if
       end if

       if (j<10) print *,j,list_ptr(j)%ep_ext_ml(1:2),i,ext_ml(i-1),filename(1:20)
     end do

!    print "(A I5 I5)",'Check nsources ',j,n_sources

    do i=1,colnr
       call release( col(i))
    end do

  end function read_set

  !--------------------------------------------------------------------
  subroutine read_Header(set, Header)
  !--------------------------------------------------------------------

    type(DataSetT), intent(in) :: set
    type(Header_type) , intent(out) :: Header

    integer :: i, j
    character(len=1) ::strBand
    character(len=2), dimension(3) :: strInst
    character(len=9) :: strEmin, strEmax

    data strInst /"PN","M1","M2"/
 
    if (hasAttribute(set,"TELESCOP") ) then
       Header%telescop = stringAttribute(set,"TELESCOP")
    else
       Header%telescop = 'UNDEFINED'
    end if
    if( hasAttribute(set,"INSTRUME") ) then
       Header%instrume = stringAttribute(set,"INSTRUME")
    else
       Header%instrume = 'UNDEFINED'
    end if
    if(  hasAttribute(set,"DETNAM") )then
       Header%detnam   = stringAttribute(set,"DETNAM")
    else
       Header%detnam   =  'UNDEFINED'
    end if
    if(  hasAttribute(set,"DATAMODE") ) then
       Header%datamode = stringAttribute(set,"DATAMODE")
    else
       Header%datamode =  'UNDEFINED'
    end if
    if(  hasAttribute(set,"OBS_MODE") ) then
       Header%obs_mode= stringAttribute(set,"OBS_MODE")
    else
       Header%obs_mode =  'UNDEFINED'
    end if
    if(  hasAttribute(set,"FILTER") ) then
       Header%filter = stringAttribute(set,"FILTER")
    else
       Header%filter =  'NONE'
    end if
    if(  hasAttribute(set,"OBSERVER")) then
       Header%observer   = stringAttribute(set,"OBSERVER")
    else
       Header%observer = 'UNDEFINED'
    end if
    if(  hasAttribute(set,"OBS_ID")) then
       Header%obs_id   = stringAttribute(set,"OBS_ID")
    else
       Header%obs_id = 'UNDEFINED'
    end if
    if( hasAttribute(set,"EXP_ID") ) then
       Header%exp_id   = stringAttribute(set,"EXP_ID")
    else
       Header%exp_id   ='UNDEFINED'
    end if
    if( hasAttribute(set,"CONTENT") ) then
       Header%content  = stringAttribute(set,"CONTENT")
    else
       Header%content  ='UNDEFINED'
    end if
    if(  hasAttribute(set,"ORIGIN") ) then
       Header%origin   = stringAttribute(set,"ORIGIN")
    else
       Header%origin   = 'UNDEFINED'
    end if
    if(  hasAttribute(set,"OBJECT")) then
       Header%object   = stringAttribute(set,"OBJECT")
    else
       Header%object   =  'UNDEFINED'
    end if
    if(  hasAttribute(set,"DATE-OBS") ) then
       Header%date_obs = stringAttribute(set,"DATE-OBS")
    else
       Header%date_obs = 'UNDEFINED'
    end if
    if(  hasAttribute(set,"DATE-END") ) then
       Header%date_end = stringAttribute(set,"DATE-END")
    else
       Header%date_end = 'UNDEFINED'
    end if
    if(  hasAttribute(set,"RADECSYS") ) then
       Header%radecsys = stringAttribute(set,"RADECSYS")
    else
       Header%radecsys = 'UNDEFINED'
    end if
    if(  hasAttribute(set, "RA_PNT") ) then
       Header%ra_pnt   = real64Attribute(set, "RA_PNT")
    else
       Header%ra_pnt   = 0.0
    end if
    if(  hasAttribute(set, "DEC_PNT") ) then
       Header%dec_pnt  = real64Attribute(set, "DEC_PNT")
    else
       Header%dec_pnt  = 0.0
    end if
    if(  hasAttribute(set, "PA_PNT") ) then
       Header%pa_pnt  = real64Attribute(set, "PA_PNT")
    else
       Header%pa_pnt  = 0.0
    end if
    if(  hasAttribute(set, "EQUINOX")) then
       Header%equinox  = real32Attribute(set, "EQUINOX")
    else
       Header%equinox  = 2000.0
    end if
    if(  hasAttribute(set,"EXPOSURE") ) then
       Header%exposure = real32Attribute(set, "EXPOSURE")
    else
       Header%exposure = 0.0
    end if

    do i=1,3
      do j=1,9
        write(strBand,'(i1)') j
        strEmin = strInst(i)//"_"//strBand//"_ELO" 
        strEmax = strInst(i)//"_"//strBand//"_EHI"
        if ( hasAttribute(set, strEmin) ) then
          Header%eblo(i,j) = real32Attribute(set, strEmin)
        else 
          Header%eblo(i,j) = -1.
        endif   
        if ( hasAttribute(set, strEmax) ) then
          Header%ebhi(i,j) = real32Attribute(set, strEmax)
        else 
          Header%ebhi(i,j) = -1.
        endif   
      end do
    end do 

  end subroutine read_Header

  !--------------------------------------------------------------------
  function read_om_set( filename, n_sources ) result( list_ptr)
  !--------------------------------------------------------------------
    type(Source_type), dimension(:), pointer :: list_ptr
    character(*), intent(in) :: filename
    integer(kind=int32) , intent (out) :: n_sources

    type(DataSetT) :: set
    type(TableT)  :: source_tab
    type(ColumnT),dimension(100) :: col

    integer(kind=int32) :: i, nrows, colnr
    integer(kind=int32),dimension(:),pointer :: src_num
    real(kind=double),dimension(:),pointer :: ra, dec
    real(kind=single),dimension(:),pointer :: radec_err



    set = dataSet(filename,READ)
    source_tab = table( set,'SRCLIST')

    ! get number of rows in this table
    nrows = numberOfRows(source_tab)
    if( nrows <= 0 ) then
       call release(source_tab) ! probably superfluous
       call release(set)        ! probably superfluous
       call error('emptyInputSet','OM source list is empty')
    end if

    colnr=0
    colnr = colnr + 1
    col(colnr) = column(source_tab, "SRCNUM",READ)
    src_num => int32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab, "RA",READ)
    ra => real64Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"DEC", READ)
    dec => real64Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = column(source_tab,"POSERR", READ)
    radec_err => real32Data( col(colnr) )

   ! allocate position list
    allocate(list_ptr(nrows))

    ! Copy the fits data into the structure
    !
    n_sources = 0
    do i=1, nrows
       ! the -1 is the diff. between F90 and C(++)

       n_sources = n_sources + 1
       list_ptr(n_sources)%src_num = src_num(i-1) 
       list_ptr(n_sources)%ra = ra(i-1) 
       list_ptr(n_sources)%dec = dec(i-1)
       list_ptr(n_sources)%radec_err = radec_err(i-1)

    end do

    do i=1,colnr
       call release( col(i))
    end do
    
  end function read_om_set

  !--------------------------------------------------------------------
  subroutine write_summary_set(filename, srclist, nsrcs, write_ra_corr, &
       header, nsets)
  !--------------------------------------------------------------------
    character(*), intent(in) :: filename
    type(Source_type), dimension(:),intent(in) :: srclist
    integer(kind=int32) , intent (in out) :: nsrcs, nsets
    logical, intent(in) :: write_ra_corr
    type(Header_type), intent(in) :: header

    type(DataSetT) :: set
    type(TableT)  :: tab
    type(ColumnT),dimension(120) :: col

    integer(kind=int32) :: i, j, colnr, ncrossid
    integer(kind=int32),dimension(:),pointer :: src_num, pn_ml_id_src
    integer(kind=int32),dimension(:),pointer :: m1_ml_id_src, m2_ml_id_src
    integer(kind=int32),dimension(:),pointer :: pn_box_id_src
    integer(kind=int32),dimension(:),pointer :: m1_box_id_src, m2_box_id_src
    integer(kind=int32),dimension(:),pointer :: ep_ml_id_src,ep_box_id_src
    real(kind=double), dimension(:), pointer :: ra, dec, bii, lii
    real(kind=double), dimension(:), pointer :: ra_corr, dec_corr
    real(kind=double), dimension(:), pointer :: bii_corr,lii_corr
    real(kind=single), dimension(:), pointer :: radec_err
    real(kind=single), dimension(:), pointer :: pn_tot,pn_tot_err
    real(kind=single), dimension(:), pointer :: m1_tot,m1_tot_err
    real(kind=single), dimension(:), pointer :: m2_tot,m2_tot_err
    real(kind=single), dimension(:), pointer :: ep_tot,ep_tot_err
    integer(kind=int32),dimension(:),pointer :: n_opt_co, om_id1
    integer(kind=int32),dimension(:),pointer :: om_id2, om_id3, om_id4,om_id5
    real(kind=single),dimension(:),pointer:: pn_xid, pn_xid_err
    real(kind=single),dimension(:),pointer:: m1_xid, m1_xid_err
    real(kind=single),dimension(:),pointer:: m2_xid, m2_xid_err
    real(kind=single),dimension(:),pointer:: ep_xid, ep_xid_err
    real(kind=single),dimension(:),pointer:: pn_det_ml, m1_det_ml, m2_det_ml
    real(kind=single),dimension(:),pointer:: ep_det_ml
    real(kind=single),dimension(:),pointer:: pn_extent, pn_ext_err,pn_ext_ml
    real(kind=single),dimension(:),pointer:: m1_extent, m1_ext_err,m1_ext_ml
    real(kind=single),dimension(:),pointer:: m2_extent, m2_ext_err,m2_ext_ml
    real(kind=single),dimension(:),pointer:: ep_extent, ep_ext_err,ep_ext_ml
    real(kind=single),dimension(:),pointer:: pn_hr1,pn_hr1_err
    real(kind=single),dimension(:),pointer:: pn_hr2,pn_hr2_err
    real(kind=single),dimension(:),pointer:: pn_hr3,pn_hr3_err 
    real(kind=single),dimension(:),pointer:: m1_hr1,m1_hr1_err
    real(kind=single),dimension(:),pointer:: m1_hr2,m1_hr2_err 
    real(kind=single),dimension(:),pointer:: m1_hr3,m1_hr3_err 
    real(kind=single),dimension(:),pointer:: m2_hr1,m2_hr1_err
    real(kind=single),dimension(:),pointer:: m2_hr2,m2_hr2_err 
    real(kind=single),dimension(:),pointer:: m2_hr3,m2_hr3_err 
    real(kind=single),dimension(:),pointer:: ep_hr1,ep_hr1_err
    real(kind=single),dimension(:),pointer:: ep_hr2,ep_hr2_err
    real(kind=single),dimension(:),pointer:: ep_hr3,ep_hr3_err 
    real(kind=single),dimension(:),pointer:: var_stat, p_var
    integer(kind=int32),dimension(:),pointer :: n_bins, var_exp_no 
    real(kind=single),dimension(:),pointer:: om_id1_dist, om_id2_dist
    real(kind=single),dimension(:),pointer::om_id3_dist,om_id4_dist,om_id5_dist
    real(kind=single) :: pn_ext_ml0,m1_ext_ml0,m2_ext_ml0,ep_ext_ml0  
    real(kind=single) :: ep_tot0, ep_xid0, ep_tot_err0, ep_xid_err0
    real(kind=single) :: pn_det_ml0,m1_det_ml0,m2_det_ml0,ep_det_ml0
    integer(kind=int32) :: nsetss
    integer(kind=int32), dimension(0:3) :: ep
    character(10),pointer :: omflag
    logical(kind=bool),dimension(:),pointer:: tseries, spectra
    logical :: sing_inst

    if( nsrcs <= 0 ) then
       call error('noData','The summary list is empty!')
    end if
    set = dataSet(filename,CREATE)
    call write_primary_header(set, header)
    tab = addTable(set,"SRCLIST",nsrcs)
    call write_summary_header(set, tab, header)

    if (header%ncrossid == 0 .and. nsets <= 3) then
      sing_inst=.TRUE.
      ep=0
      do j=1,nsets
        ep(header%id_inst(j))=ep(header%id_inst(j))+1
      end do
      do i=1,3
        if (ep(i)>1) sing_inst=.FALSE.
      end do
    else
      sing_inst=.FALSE.
    end if
     
    nsetss=nsets
    if (sing_inst) nsetss=1

    colnr=0

    colnr = colnr + 1
    col(colnr) = addColumn (tab, "SRC_NUM",Integer32)
    src_num => int32Data( col(colnr) )      
    if (.not. sing_inst) then
       colnr = colnr + 1
       col(colnr) = addColumn (tab, "EP_ML_ID_SRC",Integer32," - ",(/nsets/))
       ep_ml_id_src => int32Data( col(colnr) )      
       colnr = colnr + 1
       col(colnr) = addColumn (tab, "EP_BOX_ID_SRC",Integer32," - ",(/nsets/))
       ep_box_id_src => int32Data( col(colnr) )
    end if 

    ncrossid=MAX(header%ncrossid, 1)
    colnr = colnr + 1
    col(colnr)=addColumn(tab,"PN_ML_ID_SRC",Integer32," - ",(/ncrossid/))
    pn_ml_id_src => int32Data( col(colnr) )      
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_ML_ID_SRC",Integer32," - ",(/ncrossid/))
    m1_ml_id_src => int32Data( col(colnr) )      
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_ML_ID_SRC",Integer32," - ",(/ncrossid/))
    m2_ml_id_src => int32Data( col(colnr) )      
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_BOX_ID_SRC",Integer32," - ",(/ncrossid/))
    pn_box_id_src => int32Data( col(colnr) )      
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_BOX_ID_SRC",Integer32," - ",(/ncrossid/))
    m1_box_id_src => int32Data( col(colnr) )
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_BOX_ID_SRC",Integer32," - ",(/ncrossid/))
    m2_box_id_src => int32Data( col(colnr) )
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)

    colnr = colnr + 1
    col(colnr) = addColumn (tab, "RA",Real64, "deg")
    ra => real64Data( col(colnr) )      
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "DEC",Real64, "deg")
    dec => real64Data( col(colnr) )   
    if( write_ra_corr ) then   
       colnr = colnr + 1
       col(colnr) = addColumn (tab, "RA_CORR",Real64, "deg")
       ra_corr => real64Data( col(colnr) )      
       colnr = colnr + 1
       col(colnr) = addColumn (tab, "DEC_CORR",Real64, "deg")
       dec_corr => real64Data( col(colnr) )  
    end if
    colnr = colnr + 1
    col(colnr) = addColumn(tab,"RADEC_ERR",Real32, "arcsec")
    radec_err => real32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr) = addColumn(tab,"LII",Real64, "deg")
    lii => real64Data( col(colnr) )            
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "BII",Real64, "deg")
    bii => real64Data( col(colnr) )            
    if( write_ra_corr ) then 
       call setAttribute(tab,"RAOFFSET",header%raoffset,"arcsec",&
            "RA correction applied")
       call setAttribute(tab,"DEOFFSET",header%deoffset,"arcsec",&
            "DEC correction applied")
       call setAttribute(tab,"RAOFFERR",header%raofferr,"arcsec",&
            "Error of RA correction")
       call setAttribute(tab,"DEOFFERR",header%deofferr,"arcsec",&
            "Error of DEC correction")
       call setAttribute(tab,"ROT_CORR",header%rot_corr,"degrees",&
            "Rotation correction applied")
       call setAttribute(tab,"ROT_ERR",header%rot_err,"degrees",&
            "Error in rotation correction")
       call setAttribute(tab,"LIK_HOOD",header%lik_hood,"none",&
            "statistic")
       call setAttribute(tab,"NMATCHES",header%nmatches,"number",&
            "Number of optical-X-ray matches (eposcor)")
       colnr = colnr + 1
       col(colnr) = addColumn(tab,"LII_CORR",Real64, "deg")
       lii_corr => real64Data( col(colnr) )            
       colnr = colnr + 1
       col(colnr) = addColumn (tab, "BII_CORR",Real64, "deg")
       bii_corr => real64Data( col(colnr) )            
    end if

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_TOT",Real32,"cnts/s")
    pn_tot => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_TOT_ERR",Real32,"cnts/s")
    pn_tot_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_TOT",Real32,"cnts/s")
    m1_tot => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_TOT_ERR",Real32,"cnts/s")
    m1_tot_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_TOT",Real32,"cnts/s")
    m2_tot => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_TOT_ERR",Real32,"cnts/s")
    m2_tot_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"EP_TOT",Real32,"cnts/s")
    ep_tot => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"EP_TOT_ERR",Real32,"cnts/s")
    ep_tot_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)


    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_XID",Real32,"cnts/s")
    pn_xid => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"PN_XID_ERR",Real32,"cnts/s")
    pn_xid_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_XID",Real32,"cnts/s")
    m1_xid => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M1_XID_ERR",Real32,"cnts/s")
    m1_xid_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_XID",Real32,"cnts/s")
    m2_xid => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"M2_XID_ERR",Real32,"cnts/s")
    m2_xid_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"EP_XID",Real32,"cnts/s")
    ep_xid => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"EP_XID_ERR",Real32,"cnts/s")
    ep_xid_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_DET_ML",Real32," - ",(/nsetss/))
    pn_det_ml => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_DET_ML",Real32," - ",(/nsetss/))
    m1_det_ml => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_DET_ML",Real32," - ",(/nsetss/))
    m2_det_ml => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "EP_DET_ML",Real32," - ",(/nsetss/))
    ep_det_ml => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_EXTENT",Real32,"arcsec")
    pn_extent => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_EXT_ERR",Real32,"arcsec")
    pn_ext_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_EXT_ML",Real32,"-",(/nsetss/))
    pn_ext_ml => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_EXTENT",Real32,"arcsec")
    m1_extent => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_EXT_ERR",Real32,"arcsec")
    m1_ext_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_EXT_ML",Real32,"-",(/nsetss/))
    m1_ext_ml => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_EXTENT",Real32,"arcsec")
    m2_extent => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_EXT_ERR",Real32,"arcsec")
    m2_ext_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_EXT_ML",Real32,"-",(/nsetss/))
    m2_ext_ml => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "EP_EXTENT",Real32,"arcsec")
    ep_extent => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "EP_EXT_ERR",Real32,"arcsec")
    ep_ext_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "EP_EXT_ML",Real32,"-",(/nsetss/))
    ep_ext_ml => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)

    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_HR1",Real32,"-")
    pn_hr1 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_HR1_ERR",Real32,"-")
    pn_hr1_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_HR2",Real32,"-")
    pn_hr2 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_HR2_ERR",Real32,"-")
    pn_hr2_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_HR3",Real32,"-")
    pn_hr3 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "PN_HR3_ERR",Real32,"-")
    pn_hr3_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_HR1",Real32,"-")
    m1_hr1 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_HR1_ERR",Real32,"-")
    m1_hr1_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_HR2",Real32,"-")
    m1_hr2 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_HR2_ERR",Real32,"-")
    m1_hr2_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_HR3",Real32,"-")
    m1_hr3 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M1_HR3_ERR",Real32,"-")
    m1_hr3_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_HR1",Real32,"-")
    m2_hr1 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_HR1_ERR",Real32,"-")
    m2_hr1_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_HR2",Real32,"-")
    m2_hr2 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_HR2_ERR",Real32,"-")
    m2_hr2_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_HR3",Real32,"-")
    m2_hr3 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "M2_HR3_ERR",Real32,"-")
    m2_hr3_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "EP_HR1",Real32,"-")
    ep_hr1 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "EP_HR1_ERR",Real32,"-")
    ep_hr1_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "EP_HR2",Real32,"-")
    ep_hr2 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "EP_HR2_ERR",Real32,"-")
    ep_hr2_err => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "EP_HR3",Real32,"-")
    ep_hr3 => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr) = addColumn (tab, "EP_HR3_ERR",Real32,"-")
    ep_hr3_err => real32Data( col(colnr) )

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"VAR_STAT",Real32)
    var_stat => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"N_BINS",Integer32)
    n_bins  => int32Data(col(colnr) ) 
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"P_VAR",Real32)
    p_var => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"VAR_EXP_NO",Integer32)
    var_exp_no  => int32Data(col(colnr) ) 
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)

    !
    ! OM COLUMNS:
    !
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OM_ID1",Integer32)
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    om_id1 => int32Data( col(colnr) )
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OM_ID1_DIST",Real32,"arcsec")
    om_id1_dist => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OM_ID2",Integer32)
    om_id2 => int32Data( col(colnr) )
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OM_ID2_DIST",Real32,"arcsec")
    om_id2_dist => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OM_ID3",Integer32)
    om_id3 => int32Data( col(colnr) )
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OM_ID3_DIST",Real32,"arcsec")
    om_id3_dist => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OM_ID4",Integer32)
    om_id4 => int32Data( col(colnr) )
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OM_ID4_DIST",Real32,"arcsec")
    om_id4_dist => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OM_ID5",Integer32)
    om_id5 => int32Data( col(colnr) )
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OM_ID5_DIST",Real32,"arcsec")
    om_id5_dist => real32Data( col(colnr) )
    call setColtoNull(col(colnr),nsrcs)
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"N_OPT_CO",Integer32)
    n_opt_co => int32Data( col(colnr) )
    call setNullValue(col(colnr),-1)
    call setColtoNull(col(colnr),nsrcs)

    do i=1,nsrcs
       src_num(i-1) = srclist(i)%src_num

       if( header%ncrossid >= 1 ) then
          do j=1,header%ncrossid
             if (srclist(i)%pn_ml_id_src(j)>0) &
             pn_ml_id_src( (i-1)*ncrossid + j-1) = srclist(i)%pn_ml_id_src(j)
             if (srclist(i)%m1_ml_id_src(j)>0) &
             m1_ml_id_src( (i-1)*ncrossid + j-1) = srclist(i)%m1_ml_id_src(j)
             if (srclist(i)%m2_ml_id_src(j)>0) &
             m2_ml_id_src( (i-1)*ncrossid + j-1) = srclist(i)%m2_ml_id_src(j)
             if (srclist(i)%pn_box_id_src(j)>0) &
             pn_box_id_src( (i-1)*ncrossid + j-1) = srclist(i)%pn_box_id_src(j)
             if (srclist(i)%m1_box_id_src(j)>0) &
             m1_box_id_src( (i-1)*ncrossid + j-1) = srclist(i)%m1_box_id_src(j)
             if (srclist(i)%m2_box_id_src(j)>0) &
             m2_box_id_src( (i-1)*ncrossid + j-1) = srclist(i)%m2_box_id_src(j)
          end do
       else
          !pn_ml_id_src(i-1) = null_entry
          !m1_ml_id_src(i-1) = null_entry
          !m2_ml_id_src(i-1) = null_entry
          !pn_box_id_src(i-1) =null_entry
          !m1_box_id_src(i-1) =null_entry
          !m2_box_id_src(i-1) =null_entry
       end if

       if (sing_inst) then
          do j=1,nsets
             if (header%id_inst(j) == 1) then
             if (srclist(i)%ep_ml_id_src(j)>0) &
                pn_ml_id_src(i-1) = srclist(i)%ep_ml_id_src(j)
             if (srclist(i)%ep_box_id_src(j)>0) &
                pn_box_id_src(i-1) = srclist(i)%ep_box_id_src(j)
             end if
             if (header%id_inst(j) == 2) then
             if (srclist(i)%ep_ml_id_src(j)>0) &
                m1_ml_id_src(i-1) = srclist(i)%ep_ml_id_src(j)
             if (srclist(i)%ep_box_id_src(j)>0) &
                m1_box_id_src(i-1) = srclist(i)%ep_box_id_src(j)
             end if
             if (header%id_inst(j) == 3) then
             if (srclist(i)%ep_ml_id_src(j)>0) &
                m2_ml_id_src(i-1) = srclist(i)%ep_ml_id_src(j)
             if (srclist(i)%ep_box_id_src(j)>0) &
                m2_box_id_src(i-1) = srclist(i)%ep_box_id_src(j)
             end if
          end do         
       else     
          do j=1,nsets
             if (srclist(i)%ep_ml_id_src(j)>0) &
                ep_ml_id_src( (i-1)*nsets + j-1) = srclist(i)%ep_ml_id_src(j)
             if (srclist(i)%ep_box_id_src(j)>0) &
                ep_box_id_src( (i-1)*nsets + j-1) = srclist(i)%ep_box_id_src(j)
          end do
       end if

       ra(i-1)        = srclist(i)%ra
       dec(i-1)       = srclist(i)%dec
       if( write_ra_corr ) then 
          ra_corr(i-1)   = srclist(i)%ra_corr
          dec_corr(i-1)  = srclist(i)%dec_corr
          lii_corr(i-1)    = srclist(i)%lii_corr
          bii_corr(i-1)    = srclist(i)%bii_corr
       end if
       radec_err(i-1) = srclist(i)%radec_err
       lii(i-1)       = srclist(i)%lii
       bii(i-1)       = srclist(i)%bii
       if (srclist(i)%pn_tot>-1e30) & 
          pn_tot(i-1)      =  srclist(i)%pn_tot
       if (srclist(i)%pn_tot_err>-1e30) & 
          pn_tot_err(i-1)  =  srclist(i)%pn_tot_err
       if (srclist(i)%m1_tot>-1e30) & 
          m1_tot(i-1)      =  srclist(i)%m1_tot
       if (srclist(i)%m1_tot_err>-1e30) & 
          m1_tot_err(i-1)  =  srclist(i)%m1_tot_err
       if (srclist(i)%m2_tot>-1e30) & 
          m2_tot(i-1)      =  srclist(i)%m2_tot
       if (srclist(i)%m2_tot_err>-1e30) & 
          m2_tot_err(i-1)  =  srclist(i)%m2_tot_err
       if (sing_inst) then
         ep_tot0=0. 
         ep_tot_err0=0.
         if (srclist(i)%pn_tot>-1e30) &
            ep_tot0 = ep_tot0 + srclist(i)%pn_tot
         if (srclist(i)%pn_tot_err>-1e30) &
            ep_tot_err0 = ep_tot_err0 + srclist(i)%pn_tot_err**2 
         if (srclist(i)%m1_tot>-1e30) &
            ep_tot0 = ep_tot0 + srclist(i)%m1_tot
         if (srclist(i)%m1_tot_err>-1e30) &
            ep_tot_err0 = ep_tot_err0 + srclist(i)%m1_tot_err**2 
         if (srclist(i)%m2_tot>-1e30) &
            ep_tot0 = ep_tot0 + srclist(i)%m2_tot
         if (srclist(i)%m2_tot_err>-1e30) &
            ep_tot_err0 = ep_tot_err0 + srclist(i)%m2_tot_err**2 
         if (ep_tot0>0.) then
           ep_tot(i-1) = ep_tot0
           ep_tot_err(i-1) = sqrt(ep_tot_err0)
          end if
       else
         ep_tot(i-1)      =  srclist(i)%ep_tot
         ep_tot_err(i-1)  =  srclist(i)%ep_tot_err
       end if
       if (srclist(i)%pn_xid>-1e30) & 
          pn_xid(i-1)      =  srclist(i)%pn_xid
       if (srclist(i)%pn_xid_err>-1e30) & 
          pn_xid_err(i-1)  =  srclist(i)%pn_xid_err
       if (srclist(i)%m1_xid>-1e30) &  
          m1_xid(i-1)      =  srclist(i)%m1_xid
       if (srclist(i)%m1_xid_err>-1e30) & 
          m1_xid_err(i-1)  =  srclist(i)%m1_xid_err
       if (srclist(i)%m2_xid>-1e30) & 
          m2_xid(i-1)      =  srclist(i)%m2_xid
       if (srclist(i)%m2_xid_err>-1e30) & 
          m2_xid_err(i-1)  =  srclist(i)%m2_xid_err
       if (sing_inst) then
         ep_xid0=0.
         ep_xid_err0=0.
         if (srclist(i)%pn_xid>-1e30) &
            ep_xid0 = ep_xid0 + srclist(i)%pn_xid
         if (srclist(i)%pn_xid_err>-1e30) &
            ep_xid_err0 = ep_xid_err0 + srclist(i)%pn_xid_err**2
         if (srclist(i)%m1_xid>-1e30) &
            ep_xid0 = ep_xid0 + srclist(i)%m1_xid
         if (srclist(i)%m1_xid_err>-1e30) &
            ep_xid_err0 = ep_xid_err0 + srclist(i)%m1_xid_err**2
         if (srclist(i)%m2_xid>-1e30) &
            ep_xid0 = ep_xid0 + srclist(i)%m2_xid
         if (srclist(i)%m2_xid_err>-1e30) &
            ep_xid_err0 = ep_xid_err0 + srclist(i)%m2_xid_err**2
         if (ep_xid0>0.) then
            ep_xid(i-1) = ep_xid0
            ep_xid_err(i-1) = sqrt(ep_xid_err0)
         end if
       else
         if (srclist(i)%ep_xid>-1e30) &
            ep_xid(i-1)      =  srclist(i)%ep_xid
         if (srclist(i)%ep_xid_err>-1e30) &
            ep_xid_err(i-1)  =  srclist(i)%ep_xid_err
       end if
       if (sing_inst) then
          pn_det_ml0 = 0.
          m1_det_ml0 = 0.
          m2_det_ml0 = 0.
          ep_det_ml0 = 0.
          do j=1,nsets
             pn_det_ml0 = pn_det_ml0 + srclist(i)%pn_det_ml(j)
             m1_det_ml0 = m1_det_ml0 + srclist(i)%m1_det_ml(j)
             m2_det_ml0 = m2_det_ml0 + srclist(i)%m2_det_ml(j)
             ep_det_ml0 = ep_det_ml0 + srclist(i)%pn_det_ml(j) &
                                             + srclist(i)%m1_det_ml(j) &  
                                             + srclist(i)%m2_det_ml(j)
          end do 
          if (pn_det_ml0>0) pn_det_ml(i-1) = pn_det_ml0      
          if (m1_det_ml0>0) m1_det_ml(i-1) = m1_det_ml0      
          if (m2_det_ml0>0) m2_det_ml(i-1) = m2_det_ml0      
          if (ep_det_ml0>0) ep_det_ml(i-1) = ep_det_ml0      
       else
          do j=1,nsets
             if (srclist(i)%pn_det_ml(j)>-1e30) &
                pn_det_ml( (i-1)*nsets + j-1) = srclist(i)%pn_det_ml(j)
             if (srclist(i)%m1_det_ml(j)>-1e30) &
                m1_det_ml( (i-1)*nsets + j-1) = srclist(i)%m1_det_ml(j)
             if (srclist(i)%m2_det_ml(j)>-1e30) &
                m2_det_ml( (i-1)*nsets + j-1) = srclist(i)%m2_det_ml(j)
             if (srclist(i)%ep_det_ml(j)>-1e30) &
                ep_det_ml( (i-1)*nsets + j-1) = srclist(i)%ep_det_ml(j)
          end do
       end if
       if (srclist(i)%pn_extent>-1e30) &
          pn_extent(i-1)   =  srclist(i)%pn_extent
       if (srclist(i)%pn_ext_err>-1e30) &
          pn_ext_err(i-1)  =  srclist(i)%pn_ext_err
       if (srclist(i)%m1_extent>-1e30) &
          m1_extent(i-1)   =  srclist(i)%m1_extent
       if (srclist(i)%m1_ext_err>-1e30) &
          m1_ext_err(i-1)  =  srclist(i)%m1_ext_err
       if (srclist(i)%m2_extent>-1e30) &
          m2_extent(i-1)   =  srclist(i)%m2_extent
       if (srclist(i)%m2_ext_err>-1e30) &
          m2_ext_err(i-1)  =  srclist(i)%m2_ext_err
       if (srclist(i)%ep_extent>-1e30) &
          ep_extent(i-1)   =  srclist(i)%ep_extent
       if (srclist(i)%ep_ext_err>-1e30) &
          ep_ext_err(i-1)  =  srclist(i)%ep_ext_err
       if (sing_inst) then
          pn_ext_ml0 = 0.
          m1_ext_ml0 = 0.
          m2_ext_ml0 = 0.
          ep_ext_ml0 = 0.
          do j=1,nsets
             if (srclist(i)%pn_ext_ml(j)>-1e30) &
               pn_ext_ml0 = pn_ext_ml0 + srclist(i)%pn_ext_ml(j)
             if (srclist(i)%m1_ext_ml(j)>-1e30) &
               m1_ext_ml0 = m1_ext_ml0 + srclist(i)%m1_ext_ml(j)
             if (srclist(i)%m2_ext_ml(j)>-1e30) &
               m2_ext_ml0 = m2_ext_ml0 + srclist(i)%m2_ext_ml(j)

             if (srclist(i)%pn_ext_ml(j)>-1e30) &
               ep_ext_ml0 = ep_ext_ml0 + srclist(i)%pn_ext_ml(j)
             if (srclist(i)%m1_ext_ml(j)>-1e30) &
               ep_ext_ml0 = ep_ext_ml0 + srclist(i)%m1_ext_ml(j)
             if (srclist(i)%m2_ext_ml(j)>-1e30) &
               ep_ext_ml0 = ep_ext_ml0 + srclist(i)%m2_ext_ml(j)
          end do
          if (pn_ext_ml0>0.) pn_ext_ml(i-1)=pn_ext_ml0       
          if (m1_ext_ml0>0.) m1_ext_ml(i-1)=m1_ext_ml0       
          if (m2_ext_ml0>0.) m2_ext_ml(i-1)=m2_ext_ml0       
          if (ep_ext_ml0>0.) ep_ext_ml(i-1)=ep_ext_ml0       
       else         
          do j=1,nsets
             if (srclist(i)%pn_ext_ml(j)>-1e30) &
                pn_ext_ml( (i-1)*nsets + j-1) = srclist(i)%pn_ext_ml(j)
             if (srclist(i)%m1_ext_ml(j)>-1e30) &
                m1_ext_ml( (i-1)*nsets + j-1) = srclist(i)%m1_ext_ml(j)
             if (srclist(i)%m2_ext_ml(j)>-1e30) &
                m2_ext_ml( (i-1)*nsets + j-1) = srclist(i)%m2_ext_ml(j)
             if (srclist(i)%ep_ext_ml(j)>-1e30) &
                ep_ext_ml( (i-1)*nsets + j-1) = srclist(i)%ep_ext_ml(j)
          end do
       end if       
       if (srclist(i)%pn_hr1>-1e30)     pn_hr1(i-1)     = srclist(i)%pn_hr1
       if (srclist(i)%pn_hr1_err>-1e30) pn_hr1_err(i-1) = srclist(i)%pn_hr1_err
       if (srclist(i)%pn_hr2>-1e30)     pn_hr2(i-1)     = srclist(i)%pn_hr2
       if (srclist(i)%pn_hr2_err>-1e30) pn_hr2_err(i-1) = srclist(i)%pn_hr2_err
       if (srclist(i)%pn_hr3>-1e30)     pn_hr3(i-1)     = srclist(i)%pn_hr3
       if (srclist(i)%pn_hr3_err>-1e30) pn_hr3_err(i-1) = srclist(i)%pn_hr3_err
       if (srclist(i)%m1_hr1>-1e30)     m1_hr1(i-1)     = srclist(i)%m1_hr1
       if (srclist(i)%m1_hr1_err>-1e30) m1_hr1_err(i-1) = srclist(i)%m1_hr1_err
       if (srclist(i)%m1_hr2>-1e30)     m1_hr2(i-1)     = srclist(i)%m1_hr2
       if (srclist(i)%m1_hr2_err>-1e30) m1_hr2_err(i-1) = srclist(i)%m1_hr2_err
       if (srclist(i)%m1_hr3>-1e30)     m1_hr3(i-1)     = srclist(i)%m1_hr3
       if (srclist(i)%m1_hr3_err>-1e30) m1_hr3_err(i-1) = srclist(i)%m1_hr3_err
       if (srclist(i)%m2_hr1>-1e30)     m2_hr1(i-1)     = srclist(i)%m2_hr1
       if (srclist(i)%m2_hr1_err>-1e30) m2_hr1_err(i-1) = srclist(i)%m2_hr1_err
       if (srclist(i)%m2_hr2>-1e30)     m2_hr2(i-1)     = srclist(i)%m2_hr2
       if (srclist(i)%m2_hr2_err>-1e30) m2_hr2_err(i-1) = srclist(i)%m2_hr2_err
       if (srclist(i)%m2_hr3>-1e30)     m2_hr3(i-1)     = srclist(i)%m2_hr3
       if (srclist(i)%m2_hr3_err>-1e30) m2_hr3_err(i-1) = srclist(i)%m2_hr3_err
       if (sing_inst) then
          ep_hr1(i-1) = 0.
          ep_hr1_err(i-1) = 0.
          ep_hr2(i-1) = 0.
          ep_hr2_err(i-1) = 0.
          ep_hr3(i-1) = 0.
          ep_hr3_err(i-1) = 0.
          if (pn_hr1_err(i-1)>0.) then
             ep_hr1(i-1) = ep_hr1(i-1) + pn_hr1(i-1)/pn_hr1_err(i-1)**2
             ep_hr1_err(i-1) = ep_hr1_err(i-1) + 1./pn_hr1_err(i-1)**2
          end if
          if (m1_hr1_err(i-1)>0.) then
             ep_hr1(i-1) = ep_hr1(i-1) + m1_hr1(i-1)/m1_hr1_err(i-1)**2
             ep_hr1_err(i-1) = ep_hr1_err(i-1) + 1./m1_hr1_err(i-1)**2
          end if
          if (m2_hr1_err(i-1)>0.) then
             ep_hr1(i-1) = ep_hr1(i-1) + m2_hr1(i-1)/m2_hr1_err(i-1)**2
             ep_hr1_err(i-1) = ep_hr1_err(i-1) + 1./m2_hr1_err(i-1)**2
          end if
          if (ep_hr1_err(i-1)>0.) then
            ep_hr1(i-1) = ep_hr1(i-1)/ep_hr1_err(i-1)
            ep_hr1_err(i-1) = 1./sqrt(ep_hr1_err(i-1))
          end if

          if (pn_hr2_err(i-1)>0.) then
             ep_hr2(i-1) = ep_hr2(i-1) + pn_hr2(i-1)/pn_hr2_err(i-1)**2
             ep_hr2_err(i-1) = ep_hr2_err(i-1) + 1./pn_hr2_err(i-1)**2
          end if
          if (m1_hr2_err(i-1)>0.) then
             ep_hr2(i-1) = ep_hr2(i-1) + m1_hr2(i-1)/m1_hr2_err(i-1)**2
             ep_hr2_err(i-1) = ep_hr2_err(i-1) + 1./m1_hr2_err(i-1)**2
          end if
          if (m2_hr2_err(i-1)>0.) then
             ep_hr2(i-1) = ep_hr2(i-1) + m2_hr2(i-1)/m2_hr2_err(i-1)**2
             ep_hr2_err(i-1) = ep_hr2_err(i-1) + 1./m2_hr2_err(i-1)**2
          end if
          if (ep_hr2_err(i-1)>0.) then
            ep_hr2(i-1) = ep_hr2(i-1)/ep_hr2_err(i-1)
            ep_hr2_err(i-1) = 1./sqrt(ep_hr2_err(i-1))
          end if

          if (pn_hr3_err(i-1)>0.) then
             ep_hr3(i-1) = ep_hr3(i-1) + pn_hr3(i-1)/pn_hr3_err(i-1)**2
             ep_hr3_err(i-1) = ep_hr3_err(i-1) + 1./pn_hr3_err(i-1)**2
          end if
          if (m1_hr3_err(i-1)>0.) then
             ep_hr3(i-1) = ep_hr3(i-1) + m1_hr3(i-1)/m1_hr3_err(i-1)**2
             ep_hr3_err(i-1) = ep_hr3_err(i-1) + 1./m1_hr3_err(i-1)**2
          end if
          if (m2_hr3_err(i-1)>0.) then
             ep_hr3(i-1) = ep_hr3(i-1) + m2_hr3(i-1)/m2_hr3_err(i-1)**2
             ep_hr3_err(i-1) = ep_hr3_err(i-1) + 1./m2_hr3_err(i-1)**2
          end if     
          if (ep_hr3_err(i-1)>0.) then
            ep_hr3(i-1) = ep_hr3(i-1)/ep_hr3_err(i-1)
            ep_hr3_err(i-1) = 1./sqrt(ep_hr3_err(i-1))
          end if
       else         
          ep_hr1(i-1)      =  srclist(i)%ep_hr1 
          ep_hr1_err(i-1)  =  srclist(i)%ep_hr1_err
          ep_hr2(i-1)      =  srclist(i)%ep_hr2
          ep_hr2_err(i-1)  =  srclist(i)%ep_hr2_err
          ep_hr3(i-1)      =  srclist(i)%ep_hr3
          ep_hr3_err(i-1)  =  srclist(i)%ep_hr3_err
       end if
       ! var_stat         = 0.0
       ! p_var            = 0.0
       ! n_bins(i-1)      = 0
       ! var_exp_no       = 0
       if (srclist(i)%om_id(1)>0) &
          om_id1(i-1)      =  srclist(i)%om_id(1)
       if (srclist(i)%om_id_dist(1)>-1e30) &
          om_id1_dist(i-1) =  srclist(i)%om_id_dist(1)
       if (srclist(i)%om_id(2)>0) &
          om_id2(i-1)      =  srclist(i)%om_id(2)
       if (srclist(i)%om_id_dist(2)>-1e30) &
          om_id2_dist(i-1) =  srclist(i)%om_id_dist(2)
       if (srclist(i)%om_id(3)>0) &
          om_id3(i-1)      =  srclist(i)%om_id(3)
       if (srclist(i)%om_id_dist(3)>-1e30) &
          om_id3_dist(i-1) =  srclist(i)%om_id_dist(3)
       if (srclist(i)%om_id(4)>0) &
          om_id4(i-1)      =  srclist(i)%om_id(4)
       if (srclist(i)%om_id_dist(4)>-1e30) &
          om_id4_dist(i-1) =  srclist(i)%om_id_dist(4)
       if (srclist(i)%om_id(5)>0) &
          om_id5(i-1)      =  srclist(i)%om_id(5)
       if (srclist(i)%om_id_dist(5)>-1e30) &
          om_id5_dist(i-1) =  srclist(i)%om_id_dist(5)
       if (srclist(i)%n_opt_co>0) &
          n_opt_co(i-1)    =  srclist(i)%n_opt_co
    end do

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"OMFLAG",String," ",(/10/))
!    omflag => stringData( col(colnr))
    do i=1,nsrcs
       call setStringCell(col(colnr), i-1,  srclist(i)%omflag);
    end do

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"SPECTRA",Boolean)
    spectra => boolData( col(colnr))
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"TSERIES",Boolean)
    tseries => boolData( col(colnr))
    do i=1,nsrcs
       spectra(i-1)= .FALSE.
       tseries(i-1)= .FALSE.
    end do

    colnr = colnr + 1
    col(colnr)= addColumn(tab,"VAL_FLAG",String," ",(/12/))
    do i=1,nsrcs
       call setStringCell(col(colnr), i-1,srclist(i)%val_flag);
    end do
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"VER_FLAG",String," ",(/12/))
    do i=1,nsrcs
       call setStringCell(col(colnr), i-1,srclist(i)%ver_flag);
    end do
    colnr = colnr + 1
    col(colnr)= addColumn(tab,"VER_COMM",String," ",(/40/))
    do i=1,nsrcs
       call setStringCell(col(colnr), i-1,srclist(i)%ver_comm);
    end do


    do i=1, colnr
       call release(col(i))
    end do

    call release(tab)
    call release(set)

  end subroutine write_summary_set

  !---------------------------------------------------------------------
  subroutine setColtoNull (col, nsrcs)
  !---------------------------------------------------------------------

     type(ColumnT), intent(in) :: col 
     integer, intent(in) :: nsrcs
     integer :: i

     do i=0,nsrcs-1
        call settoNull(col,i)     
     end do
 
  end subroutine setColtoNull 

  !--------------------------------------------------------------------
  subroutine write_primary_header(set, header)
  !--------------------------------------------------------------------  
    type(DataSetT), intent(in out) :: set
    type(Header_type) , intent(in) :: header

    call setAttribute(set,"TELESCOP", header%telescop)
    call setAttribute(set,'INSTRUME', header%instrume)
    call setAttribute(set,'DETNAM', header%detnam)
    call setAttribute(set,'DATAMODE', header%datamode)
    call setAttribute(set,'FILTER', header%filter)
    call setAttribute(set,'OBS_MODE', header%obs_mode)
    call setAttribute(set,'OBS_ID', header%obs_id)
    call setAttribute(set,'EXP_ID', header%exp_id)
    call setAttribute(set,'OBJECT', header%object)
    call setAttribute(set,'RA_PNT', header%ra_pnt)
    call setAttribute(set,'DEC_PNT', header%dec_pnt)
    call setAttribute(set,'PA_PNT', header%pa_pnt)
    call setAttribute(set,'RADECSYS', header%radecsys)
    call setAttribute(set,'EQUINOX', header%equinox)
    call setAttribute(set,'EXPOSURE', header%exposure)
    call setAttribute(set,'DATE-OBS', header%date_obs)
    call setAttribute(set,'DATE-END', header%date_end)
    call setAttribute(set,'OBSERVER', header%observer)

  end subroutine write_primary_header

  !--------------------------------------------------------------------
  subroutine write_summary_header(set,tab, header)
  !--------------------------------------------------------------------  
    type(DataSetT), intent(in out) :: set
    type(TableT)  :: tab
    type(Header_type) , intent(in) :: header


    character(6) :: setname='EP_SET'
    character(7) :: crossidname='CROSSID'
    character(15) :: set_id 
    character(len=1) ::strBand
    character(len=2), dimension(3) :: strInst
    character(len=9) :: strEmin, strEmax
    integer(kind=int32)  :: i,j

    data strInst /"PN","M1","M2"/

    call setAttribute(set,"CONTENT", "EPIC CAMERA ML SOURCE LIST")

    call setAttribute(tab,'NSETS',header%nsets," ",&
         "Number of input data sets.")
    do i=1,header%nsets
       if( i <= 9) write (set_id,'(A6I1)') setname,i
       if( i > 9) write (set_id,'(A6I2)') setname,i
       call setAttribute(tab,set_id, header%inputsets(i))
    end do
    call setAttribute(tab,'NCROSSID', header%ncrossid," ","Number of cross id sets (earlier stage products).")
    do i=1,header%ncrossid
       write (set_id,'(A7I1)') crossidname,i
       call setAttribute(tab,set_id, header%crossidsets(i))
    end do

    call setAttribute(tab,"SYST_ERR", header%syst_err,&
         "arcsec","Systematic error for EPIC source matching")
    call setAttribute(tab,"MAX_ERR", header%max_err,"sigmas",&
         "Maximum error for correlating sources")
    call setAttribute(tab,"OMCROSS", header%omcross,&
         "Are X-ray  cross correlated with OM sources?")
    call setAttribute(tab,"OM_DIST", header%om_dist,"arcsec",&
         "Maximum distance for OM sources")

    do i=1,3
      do j=1,9
        if (header%eblo(i,j) >0.) then
          write(strBand,'(i1)') j
          strEmin = strInst(i)//"_"//strBand//"_ELO" 
          call setAttribute(tab,strEmin,header%eblo(i,j))
        end if
        if (header%ebhi(i,j) >0.) then
          write(strBand,'(i1)') j
          strEmax = strInst(i)//"_"//strBand//"_ELO" 
          call setAttribute(tab,strEmax,header%ebhi(i,j))
        end if
      end do
    end do 
    
  end subroutine write_summary_header


  !--------------------------------------------------------------------
  subroutine write_html_list(filename,srclist, nsrcs, header, nsets)
  !--------------------------------------------------------------------  
! D.M. 
! D.M. Changed to reduce the html table massively (by ~ 75% - 80%) to just give
! D.M. an overview of the identified sources, for details see FITS table !
! D.M. 
    character(*), intent(in) :: filename
    type(Source_type), dimension(:),intent(inout) :: srclist
    integer(kind=int32) , intent (in) :: nsrcs, nsets
    type(Header_type) , intent(in) :: header
    integer(kind=int32) :: i,j,ip,left
    character(35) :: begcell='<TD ALIGN=middle><FONT SIZE=-3>'
    character(25) :: endcell='</FONT></TD>'
    character(5) :: seperator=' , '
    character(10) :: forma
    character(200) :: line

    real(kind=single) :: ep_tot0, ep_tot_err0, &
                         ep_xid0, ep_xid_err0, ep_ext_ml0
    integer(kind=int32), dimension(0:3) :: ep
    logical :: sing_inst

    open(unit=7,file=filename,form="formatted",status="unknown")
    
    if (header%ncrossid == 0 .and. nsets <= 3) then
      sing_inst=.TRUE.
      ep=0
      do j=1,nsets
        ep(header%id_inst(j))=ep(header%id_inst(j))+1
      end do
      do i=1,3
        if (ep(i)>1) sing_inst=.FALSE.
      end do
    else
      sing_inst=.FALSE.
    end if

    write (7,"(a)") "<HTML><HEAD>"
    write (7,"(a)") &
     "<TITLE>EPIC HTML summary source list</TITLE>"
    write (7,"(a)") "</head>"
    write (7,"(a)") "<BODY><CENTER>"
    write (7,"(a)") "<H2>Final EPIC observation source list</H2>"
!    write (7,"(a)") "<TABLE BORDER=-0.5>"

    write (7,"(a)") "<PRE>"
! D.M.     write (7,"(a,a,a)") begcell,"<B>SRC_NUM</B><BR>-",endcell

    do ip=1,200
      line(ip:ip)=" "
    end do 
    ip=1 

    write (line(ip:),"(a)") "  SRC_NUM "
    ip=ip+10
    if (.not. sing_inst) then 
       write (line(ip:),"(a)") " EP_ML_ID " 
       ip=ip+10*header%nsets
       write (line(ip:),"(a)") " EP_BOX_ID " 
       ip=ip+10*header%nsets
!      write (7,"(a,a,a)") begcell,"<B>EP_ML_ID_SRC</B><BR>-",endcell
!      write (7,"(a,a,a)") begcell,"<B>EP_BOX_ID_SRC</B><BR>-",endcell
    else      
       write (line(ip:),"(3a)") " PN_ML_ID "," M1_ML_ID "," M2_ML_ID " 
       ip=ip+30
       write (line(ip:),"(3a)") " PN_BOX_ID"," M1_BOX_ID"," M2_BOX_ID"
       ip=ip+30
!      write (7,"(a,a,a)") begcell,"<B>PN_ML_ID_SRC</B><BR>-",endcell
!      write (7,"(a,a,a)") begcell,"<B>M1_ML_ID_SRC</B><BR>-",endcell
!      write (7,"(a,a,a)") begcell,"<B>M2_ML_ID_SRC</B><BR>-",endcell
!      write (7,"(a,a,a)") begcell,"<B>PN_BOX_ID_SRC</B><BR>-",endcell
!      write (7,"(a,a,a)") begcell,"<B>M1_BOX_ID_SRC</B><BR>-",endcell
!      write (7,"(a,a,a)") begcell,"<B>M2_BOX_ID_SRC</B><BR>-",endcell
    end if
    write(line(ip:),"(3a)")"    RA    ","    DEC   "," RADEC_ERR"
    ip=ip+30 
!    write (7,"(a,a,a)") begcell,"<B>RA</B><BR>(deg)",endcell
!    write (7,"(a,a,a)") begcell,"<B>DEC</B><BR>(deg)", endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>RA_CORR</B><BR>(deg)",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>DEC_CORR</B><BR>(deg)",endcell
!    write (7,"(a,a,a)") begcell,"<B>RADEC_ERR</B><BR>(arcsec)",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>LII</B><BR>(deg)",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>BII</B><BR>(deg)</FONT></TH>"
! D.M.     write (7,"(a,a,a)") begcell,"<B>LII_CORR</B><BR>(deg)", endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>BII_CORR</B><BR>(deg)",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>PN_TOT</B><BR>(cnts/s)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>PN_TOT_ERR</B><BR>(cnts/s)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M1_TOT</B><BR>(cnts/s)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M1_TOT_ERR</B><BR>(cnts/s)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M2_TOT</B><BR>(cnts/s)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M2_TOT_ERR</B><BR>(cnts/s)</B>",endcell
    write(line(ip:),"(2a)")"  EP_TOT  ","EP_TOT_ERR"
    ip=ip+20 
!    write (7,"(a,a,a)") begcell,"<B>EP_TOT</B><BR>(cnts/s)</B>",endcell
!    write (7,"(a,a,a)") begcell,"<B>EP_TOT_ERR</B><BR>(cnts/s)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>PN_XID</B><BR>(cnts/s)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>PN_XID_ERR</B><BR>(cnts/s)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M1_XID</B><BR>(cnts/s)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M1_XID_ERR</B><BR>(cnts/s)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M2_XID</B><BR>(cnts/s)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M2_XID_ERR</B><BR>(cnts/s)</B>",endcell
    write(line(ip:),"(2a)")"  EP_XID  ","EP_XID_ERR"
    ip=ip+20 
!    write (7,"(a,a,a)") begcell,"<B>EP_XID</B><BR>(cnts/s)</B>",endcell
!    write (7,"(a,a,a)") begcell,"<B>EP_XID_ERR</B><BR>(cnts/s)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>PN_DET_ML</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M1_DET_ML</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M2_DET_ML</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>EP_DET_ML</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>PN_EXTENT</B><BR>(arcsec)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>PN_EXT_ERR</B><BR>(arcsec)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>PN_EXT_ML</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M1_EXTENT</B><BR>(arcsec)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M1_EXT_ERR</B><BR>(arcsec)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M1_EXT_ML</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M2_EXTENT</B><BR>(arcsec)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M2_EXT_ERR</B><BR>(arcsec)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M2_EXT_ML</B><BR>-</B>",endcell
    if (sing_inst) then
      write(line(ip:),"(3a)")"  EP_EXT  ","EP_EXT_ERR"," EP_EXT_ML"
      ip=ip+30
    else
      write(line(ip:),"(3a)")"  EP_EXT  ","EP_EXT_ERR"
      ip=ip+20
      write(line(ip:),"(3a)")" EP_EXT_ML"
      ip=ip+10*header%nsets
    end if      
!    write (7,"(a,a,a)") begcell,"<B>EP_EXTENT</B><BR>(arcsec)</B>",endcell
!    write (7,"(a,a,a)") begcell,"<B>EP_EXT_ERR</B><BR>(arcsec)</B>",endcell
!    write (7,"(a,a,a)") begcell,"<B>EP_EXT_ML</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>PN_HR1</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>PN_HR1_ERR</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>PN_HR2</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>PN_HR2_ERR</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>PN_HR3</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>PN_HR3_ERR</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M1_HR1</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M1_HR1_ERR</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M1_HR2</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M1_HR2_ERR</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M1_HR3</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M1_HR3_ERR</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M2_HR1</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M2_HR1_ERR</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M2_HR2</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M2_HR2_ERR</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M2_HR3</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>M2_HR3_ERR</B><BR>-</B>",endcell
!    write (7,"(a,a,a)") begcell,"<B>EP_HR1</B><BR>-</B>",endcell
!    write (7,"(a,a,a)") begcell,"<B>EP_HR1_ERR</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>EP_HR2</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>EP_HR2_ERR</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>EP_HR3</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>EP_HR3_ERR</B><BR>-</B>",endcell
!    write (7,"(a,a,a)") begcell,"<B>VAR_STAT</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>N_BINS</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>P_VAR</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>VAR_EXP_NO</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>OM_ID1</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>OM_ID1_DIST</B><BR>(arcsec)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>OM_ID2</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>OM_ID2_DIST</B><BR>(arcsec)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>OM_ID3</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>OM_ID3_DIST</B><BR>(arcsec)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>OM_ID4</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>OM_ID4_DIST</B><BR>(arcsec)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>OM_ID5</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>OM_ID5_DIST</B><BR>(arcsec)</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>N_OPT_CO</B><BR>-</B>",endcell
    write (line(ip:),"(a)")"  OMFLAG  "
    ip=ip+10
!    write (7,"(a,a,a)") begcell,"<B>OMFLAG</B><BR>-</B>",endcell
!    write (7,"(a,a,a)") begcell,"<B>SPECTRA</B><BR>-</B>",endcell
!    write (7,"(a,a,a)") begcell,"<B>TSERIES</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>VAL_FLAG</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>VER_FLAG</B><BR>-</B>",endcell
! D.M.     write (7,"(a,a,a)") begcell,"<B>VER_COMM</B><BR>-</B>",endcell

     write (7,"(a,/)")line(1:ip)
!    write (7,"(a)") "</TR>"

    do i=1,nsrcs
!      write(7,"(a)")line(1:30)
      do ip=1,200
        line(ip:ip)=" "
      end do 
      ip=1 
!       write (7,"(a)") "<TR>"
! D.M.        write (7,"(a,i4)") begcell, srclist(i)%src_num, endcell
! D.M.        write (7,"(a)") begcell
! D.M.        do j=1,header%ncrossid
! D.M.           write (7,"(i4)")  srclist(i)%pn_ml_id_src(j)
! D.M.           if( j /= header%ncrossid)  write (7,"(a)") seperator
! D.M.        end do
! D.M.        write (7,"(a)") endcell
! D.M.        write (7,"(a)") begcell
! D.M.        do j=1,header%ncrossid
! D.M.           write (7,"(i4)")  srclist(i)%m1_ml_id_src(j)
! D.M.           if( j /= header%ncrossid)  write (7,"(a)") seperator
! D.M.        end do
! D.M.        write (7,"(a)") endcell
! D.M.        write (7,"(a)") begcell
! D.M.        do j=1,header%ncrossid
! D.M.           write (7,"(i4)")  srclist(i)%m2_ml_id_src(j)
! D.M.           if( j /= header%ncrossid)  write (7,"(a)") seperator
! D.M.        end do
! D.M.        write (7,"(a)") endcell

       
       left = int(log10(real(i)))+1
       write (forma,"(a,i1,a)") "(a,i",left,",a)" 
       write (line(ip:),forma) '<A NAME="SRC_',i,'">'
       ip=ip+15+left

       write(line(ip:),"(i7,3x)")i
       ip=ip+10

       if (sing_inst) then
         do j=1,nsets
            if (header%id_inst(j) == 1) then
            if (srclist(i)%ep_ml_id_src(j)>0) &
               srclist(i)%pn_ml_id_src(1) = srclist(i)%ep_ml_id_src(j)
            if (srclist(i)%ep_box_id_src(j)>0) &
               srclist(i)%pn_box_id_src(1) = srclist(i)%ep_box_id_src(j)
            end if
            if (header%id_inst(j) == 2) then
            if (srclist(i)%ep_ml_id_src(j)>0) &
               srclist(i)%m1_ml_id_src(1) = srclist(i)%ep_ml_id_src(j)
            if (srclist(i)%ep_box_id_src(j)>0) &
               srclist(i)%m1_box_id_src(1) = srclist(i)%ep_box_id_src(j)
            end if
            if (header%id_inst(j) == 3) then
            if (srclist(i)%ep_ml_id_src(j)>0) &
               srclist(i)%m2_ml_id_src(1) = srclist(i)%ep_ml_id_src(j)
            if (srclist(i)%ep_box_id_src(j)>0) &
               srclist(i)%m2_box_id_src(1) = srclist(i)%ep_box_id_src(j)
            end if
          end do
          if (srclist(i)%pn_ml_id_src(1)>0) then         
            write(line(ip:),"(i7,3x)")srclist(i)%pn_ml_id_src(1)
            ip=ip+10
!            write (7,"(a,i4,a)") begcell,srclist(i)%pn_ml_id_src(1),endcell
          else
            write(line(ip:),"(a)")"      -   "
            ip=ip+10
!            write (7,"(3a)") begcell," - ",endcell
          end if
          if (srclist(i)%m1_ml_id_src(1)>0) then 
            write(line(ip:),"(i7,3x)")srclist(i)%m1_ml_id_src(1)
            ip=ip+10
!            write (7,"(a,i4,a)") begcell,srclist(i)%m1_ml_id_src(1),endcell
          else
            write(line(ip:),"(a)")"      -   "
            ip=ip+10
!            write (7,"(3a)") begcell," - ",endcell
          end if
          if (srclist(i)%m2_ml_id_src(1)>0) then     
            write(line(ip:),"(i7,3x)")srclist(i)%m2_ml_id_src(1)
            ip=ip+10
!            write (7,"(a,i4,a)") begcell,srclist(i)%m2_ml_id_src(1),endcell
          else
            write(line(ip:),"(a)")"      -   "
            ip=ip+10
!            write (7,"(3a)") begcell," - ",endcell
          end if
          if (srclist(i)%pn_box_id_src(1)>0) then 
            write(line(ip:),"(i7,3x)")srclist(i)%pn_box_id_src(1)
            ip=ip+10
!            write (7,"(a,i4,a)") begcell,srclist(i)%pn_box_id_src(1),endcell
          else
            write(line(ip:),"(a)")"      -   "
            ip=ip+10
!            write (7,"(3a)") begcell," - ",endcell
          end if
          if (srclist(i)%m1_box_id_src(1)>0) then
            write(line(ip:),"(i7,3x)")srclist(i)%m1_box_id_src(1)
            ip=ip+10
!            write (7,"(a,i4,a)") begcell,srclist(i)%m1_box_id_src(1),endcell
          else
            write(line(ip:),"(a)")"      -   "
            ip=ip+10
!            write (7,"(3a)") begcell," - ",endcell
          end if
          if (srclist(i)%m2_box_id_src(1)>0) then
            write(line(ip:),"(i7,3x)")srclist(i)%m2_box_id_src(1)
            ip=ip+10
!            write (7,"(a,i4,a)") begcell,srclist(i)%m2_box_id_src(1),endcell
          else
            write(line(ip:),"(a)")"      -   "
            ip=ip+10
!            write (7,"(3a)") begcell," - ",endcell
          end if
       else
!          write (7,"(a)") begcell
          do j=1,header%nsets
             write(line(ip:),"(i7,3x)") srclist(i)%ep_ml_id_src(j)
             ip=ip+10
!             write (7,"(i4)") srclist(i)%ep_ml_id_src(j)
!             if( j /= header%nsets)  write (7,"(a)") seperator
          end do
!          write (7,"(a)") endcell
!          write (7,"(a)") begcell
          do j=1,header%nsets
             write(line(ip:),"(i7,3x)") srclist(i)%ep_box_id_src(j)
             ip=ip+10
!             write (7,"(i4)") srclist(i)%ep_box_id_src(j)
!             if( j /= header%nsets)  write (7,"(a)") seperator
          end do
!          write (7,"(a)") endcell
       end if
       write (line(ip:),"(2f10.5)")srclist(i)%ra,srclist(i)%dec
       ip=ip+20
!       write (7,"(a,f10.5,a)")  begcell,srclist(i)%ra, endcell
!       write (7,"(a,f10.5,a)")  begcell, srclist(i)%dec, endcell
! D.M.        write (7,"(a,f10.5,a)")  begcell, srclist(i)%ra_corr, endcell
! D.M.        write (7,"(a,f10.5,a)")  begcell, srclist(i)%dec_corr, endcell
       write (line(ip:),"(f8.2,2x)")srclist(i)%radec_err
       ip=ip+10
!       write (7,"(a,f6.2,a)")   begcell, srclist(i)%radec_err, endcell
! D.M.        write (7,"(a,f10.5,a)")  begcell, srclist(i)%lii, endcell
! D.M.        write (7,"(a,f10.5,a)")  begcell, srclist(i)%bii, endcell
! D.M.        write (7,"(a,f10.5,a)")  begcell, srclist(i)%lii_corr, endcell
! D.M.        write (7,"(a,f10.5,a)")  begcell, srclist(i)%bii_corr, endcell
! D.M.        write (7,"(a,f10.5,a)")  begcell, srclist(i)%pn_tot, endcell
! D.M.        write (7,"(a,f10.5,a)")  begcell, srclist(i)%pn_tot_err, endcell
! D.M.        write (7,"(a,f10.5,a)")  begcell, srclist(i)%m1_tot, endcell
! D.M.        write (7,"(a,f10.5,a)")  begcell, srclist(i)%m1_tot_err, endcell
! D.M.        write (7,"(a,f10.5,a)")  begcell, srclist(i)%m2_tot, endcell
! D.M.        write (7,"(a,f10.5,a)")  begcell, srclist(i)%m2_tot_err, endcell
       if (sing_inst) then
          ep_tot0=0. 
          ep_tot_err0=0.
          if (srclist(i)%pn_tot>-1e30) &
             ep_tot0 = ep_tot0 + srclist(i)%pn_tot
          if (srclist(i)%pn_tot_err>-1e30) &
             ep_tot_err0 = ep_tot_err0 + srclist(i)%pn_tot_err**2 
          if (srclist(i)%m1_tot>-1e30) &
             ep_tot0 = ep_tot0 + srclist(i)%m1_tot
          if (srclist(i)%m1_tot_err>-1e30) &
             ep_tot_err0 = ep_tot_err0 + srclist(i)%m1_tot_err**2 
          if (srclist(i)%m2_tot>-1e30) &
             ep_tot0 = ep_tot0 + srclist(i)%m2_tot
          if (srclist(i)%m2_tot_err>-1e30) &
             ep_tot_err0 = ep_tot_err0 + srclist(i)%m2_tot_err**2 
          if (ep_tot0>0.) then
             srclist(i)%ep_tot = ep_tot0
             srclist(i)%ep_tot_err = sqrt(ep_tot_err0)
          end if
       end if       
       write(line(ip:),"(2(f9.5,1x))") srclist(i)%ep_tot,srclist(i)%ep_tot_err
       ip=ip+20 
!       write (7,"(a,f10.5,a)")  begcell, srclist(i)%ep_tot, endcell
!       write (7,"(a,f10.5,a)")  begcell, srclist(i)%ep_tot_err, endcell
! D.M.        write (7,"(a,f10.5,a)")  begcell, srclist(i)%pn_xid, endcell
! D.M.        write (7,"(a,f10.5,a)")  begcell, srclist(i)%pn_xid_err, endcell
! D.M.        write (7,"(a,f10.5,a)")  begcell, srclist(i)%m1_xid, endcell
! D.M.        write (7,"(a,f10.5,a)")  begcell, srclist(i)%m1_xid_err, endcell
! D.M.        write (7,"(a,f10.5,a)")  begcell, srclist(i)%m2_xid, endcell
! D.M.        write (7,"(a,f10.5,a)")  begcell, srclist(i)%m2_xid_err, endcell
      if (sing_inst) then
         ep_xid0=0.
         ep_xid_err0=0.
         if (srclist(i)%pn_xid>-1e30) &
            ep_xid0 = ep_xid0 + srclist(i)%pn_xid
         if (srclist(i)%pn_xid_err>-1e30) &
            ep_xid_err0 = ep_xid_err0 + srclist(i)%pn_xid_err**2
         if (srclist(i)%m1_xid>-1e30) &
            ep_xid0 = ep_xid0 + srclist(i)%m1_xid
         if (srclist(i)%m1_xid_err>-1e30) &
            ep_xid_err0 = ep_xid_err0 + srclist(i)%m1_xid_err**2
         if (srclist(i)%m2_xid>-1e30) &
            ep_xid0 = ep_xid0 + srclist(i)%m2_xid
         if (srclist(i)%m2_xid_err>-1e30) &
            ep_xid_err0 = ep_xid_err0 + srclist(i)%m2_xid_err**2
         if (ep_xid0>0.) then
            srclist(i)%ep_xid = ep_xid0
            srclist(i)%ep_xid_err = sqrt(ep_xid_err0)
         end if
       end if
       if (srclist(i)%ep_xid_err>1e-30) then 
         write(line(ip:),"(2(f9.5,1x))") srclist(i)%ep_xid,srclist(i)%ep_xid_err
         ip=ip+20
       else
         write(line(ip:),"(2a)")"      -   ","      -   "
         ip=ip+20
       end if 
!       write (7,"(a,f10.5,a)")  begcell, srclist(i)%ep_xid, endcell
!       write (7,"(a,f10.5,a)")  begcell, srclist(i)%ep_xid_err, endcell
! D.M.        write (7,"(a)") begcell
! D.M.        do j=1,header%nsets
! D.M.           write (7,"(f9.1)")  srclist(i)%pn_det_ml(j)
! D.M.           if( j /= header%nsets)  write (7,"(a)") seperator
! D.M.        end do
! D.M.        write (7,"(a)") endcell
! D.M.        write (7,"(a)") begcell
! D.M.        do j=1,header%nsets
! D.M.           write (7,"(f9.1)")  srclist(i)%m1_det_ml(j)
! D.M.           if( j /= header%nsets)  write (7,"(a)") seperator
! D.M.        end do
! D.M.        write (7,"(a)") endcell
! D.M.        write (7,"(a)") begcell
! D.M.        do j=1,header%nsets
! D.M.           write (7,"(f9.1)")  srclist(i)%m2_det_ml(j)
! D.M.           if( j /= header%nsets)  write (7,"(a)") seperator
! D.M.        end do
! D.M.        write (7,"(a)") endcell
! D.M.        write (7,"(a)") begcell
! D.M.        do j=1,header%nsets
! D.M.           write (7,"(f9.1)")  srclist(i)%ep_det_ml(j)
! D.M.           if( j /= header%nsets)  write (7,"(a)") seperator
! D.M.        end do
! D.M.        write (7,"(a)") endcell
! D.M. 
! D.M.        write (7,"(a,f6.1,a)")  begcell, srclist(i)%pn_extent, endcell
! D.M.        write (7,"(a,f6.1,a)")  begcell, srclist(i)%pn_ext_err, endcell
! D.M.        write (7,"(a)") begcell
! D.M.        do j=1,header%nsets
! D.M.           write (7,"(f6.1)")  srclist(i)%pn_ext_ml(j)
! D.M.           if( j /= header%nsets)  write (7,"(a)") seperator
! D.M.        end do
! D.M.        write (7,"(a)") endcell
! D.M.        write (7,"(a,f6.1,a)")  begcell, srclist(i)%m1_extent, endcell
! D.M.        write (7,"(a,f6.1,a)")  begcell, srclist(i)%m1_ext_err, endcell
! D.M.        write (7,"(a)") begcell
! D.M.        do j=1,header%nsets
! D.M.           write (7,"(f6.1)")  srclist(i)%m1_ext_ml(j)
! D.M.           if( j /= header%nsets)  write (7,"(a)") seperator
! D.M.        end do
! D.M.        write (7,"(a)") endcell
! D.M.        write (7,"(a,f6.1,a)")  begcell, srclist(i)%m2_extent, endcell
! D.M.        write (7,"(a,f6.1,a)")  begcell, srclist(i)%m2_ext_err, endcell
! D.M.        write (7,"(a)") begcell
! D.M.        do j=1,header%nsets
! D.M.           write (7,"(f6.1)")  srclist(i)%m2_ext_ml(j)
! D.M.           if( j /= header%nsets)  write (7,"(a)") seperator
! D.M.        end do
! D.M.        write (7,"(a)") endcell
       if (srclist(i)%ep_extent>0.) then
         write(line(ip:),"(f8.1,2x)")srclist(i)%ep_extent
         ip=ip+10
!         write (7,"(a,f6.1,a)")  begcell, srclist(i)%ep_extent, endcell
       else
         write(line(ip:),"(a)")"      -   "
         ip=ip+10
!         write (7,"(3a)") begcell," - ",endcell
       end if
       if (srclist(i)%ep_ext_err>0.) then
         write(line(ip:),"(f8.1,2x)") srclist(i)%ep_ext_err
         ip=ip+10
!         write (7,"(a,f6.1,a)")  begcell, srclist(i)%ep_ext_err, endcell
       else
         write(line(ip:),"(a)")"      -   "
         ip=ip+10
!         write (7,"(3a)") begcell," - ",endcell
       end if
       if (sing_inst) then
          ep_ext_ml0 = 0.
          do j=1,nsets
             if (srclist(i)%pn_ext_ml(j)>-1e30) &
               ep_ext_ml0 = ep_ext_ml0 + srclist(i)%pn_ext_ml(j)
             if (srclist(i)%m1_ext_ml(j)>-1e30) &
               ep_ext_ml0 = ep_ext_ml0 + srclist(i)%m1_ext_ml(j)
             if (srclist(i)%m2_ext_ml(j)>-1e30) &
               ep_ext_ml0 = ep_ext_ml0 + srclist(i)%m2_ext_ml(j)
          end do
          if (ep_ext_ml0>0.) srclist(i)%ep_ext_ml(1)=ep_ext_ml0
          if (srclist(i)%ep_ext_ml(1)>0.) then 
            write(line(ip:),"(f8.1,2x)")srclist(i)%ep_ext_ml(1)
            ip=ip+10
!            write (7,"(a,f6.1,a)") begcell,srclist(i)%ep_ext_ml(j),endcell
          else
            write(line(ip:),"(a)")"      -   "
            ip=ip+10
!            write (7,"(3a)") begcell," - ",endcell
          end if
       else      
!          write (7,"(a)") begcell
          do j=1,header%nsets
             write(line(ip:),"(f8.1,2x)")srclist(i)%ep_ext_ml(j)
             ip=ip+10
!             write (7,"(f6.1)")  srclist(i)%ep_ext_ml(j)
!             if( j /= header%nsets)  write (7,"(a)") seperator
          end do
!          write (7,"(a)") endcell
       end if
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%pn_hr1, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%pn_hr1_err, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%pn_hr2, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%pn_hr2_err, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%pn_hr3, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%pn_hr3_err, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%m1_hr1, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%m1_hr1_err, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%m1_hr2, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%m1_hr2_err, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%m1_hr3, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%m1_hr3_err, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%m2_hr1, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%m2_hr1_err, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%m2_hr2, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%m2_hr2_err, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%m2_hr3, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%m2_hr3_err, endcell
!       write (7,"(a,f10.3,a)") begcell, srclist(i)%ep_hr1, endcell
!       write (7,"(a,f10.3,a)") begcell, srclist(i)%ep_hr1_err, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%ep_hr2, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%ep_hr2_err, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%ep_hr3, endcell
! D.M.        write (7,"(a,f10.3,a)") begcell, srclist(i)%ep_hr3_err, endcell
!       write (7,"(a,f8.3,a)") begcell,srclist(i)%var_stat,endcell
! D.M.        write (7,"(a,i4,a)") begcell,srclist(i)%n_bins,endcell
! D.M.        write (7,"(a,f10.4,a)") begcell,srclist(i)%p_var,endcell
! D.M.        write (7,"(a,i4,a)") begcell,srclist(i)%var_exp_no,endcell
! D.M. 
! D.M.        do j=1,5
! D.M.           write (7,"(a,i4,a)")  begcell,srclist(i)%om_id(j),endcell
! D.M.           write (7,"(a,f6.2,a)")  begcell,srclist(i)%om_id_dist(j),endcell
! D.M.        end do
! D.M.        write (7,"(a,i4,a)")  begcell, srclist(i)%n_opt_co, endcell
       write(line(ip:),"(a)")srclist(i)%omflag
       ip=ip+10
!       write (7,"(a,a,a)")  begcell, srclist(i)%omflag, endcell
!       if( srclist(i)%spectra ) then
!          write (7,"(a,a3,a)")  begcell, "T", endcell
!       else
!          write (7,"(a,a3,a)")  begcell, "F", endcell
!       end if
!       if( srclist(i)%tseries ) then
!          write (7,"(a,a3,a)")  begcell, "T", endcell
!       else
!          write (7,"(a,a3,a)")  begcell, "F", endcell
!       end if
! D.M.        write (7,"(a,a,a)")  begcell, srclist(i)%val_flag, endcell
! D.M.        write (7,"(a,a,a)")  begcell, srclist(i)%ver_flag, endcell
! D.M.        write (7,"(a,a,a)")  begcell, srclist(i)%ver_comm, endcell

        write(7,"(a)")line(1:ip) 
!       write (7,"(a)") "</TR>"

    end do
    write (7,"(a)") "</PRE>"
    write (7,"(a)") "</FONT></TABLE>"
    write (7,"(a)") "</CENTER></BODY></HTML>"

    close (7)
    
  end subroutine write_html_list
       
  !--------------------------------------------------------------------
  subroutine init_list(srclist, nsrcs)
  !--------------------------------------------------------------------  
    type(Source_type), dimension(:), intent(in out) :: srclist
    integer(kind=int32),intent(in) :: nsrcs
    integer(kind=int32) :: i,j

    do i=1,nsrcs
     srclist(i)%id_src = null_entry
     srclist(i)%id_inst = null_entry
     srclist(i)%id_band = null_entry
     srclist(i)%src_num = null_entry
     do j=1, max_nsets
        srclist(i)%ep_ml_id_src(j) = null_entry
        srclist(i)%pn_ml_id_src(j) = null_entry
        srclist(i)%m1_ml_id_src(j) = null_entry
        srclist(i)%m2_ml_id_src(j) = null_entry
        srclist(i)%ep_box_id_src(j) = null_entry 
        srclist(i)%pn_box_id_src(j) = null_entry 
        srclist(i)%m1_box_id_src(j) = null_entry
        srclist(i)%m2_box_id_src(j) = null_entry
        srclist(i)%pn_det_ml(j) = 0.0
        srclist(i)%m1_det_ml(j) = 0.0 
        srclist(i)%m2_det_ml(j) = 0.0  
        srclist(i)%ep_det_ml(j) = 0.0 
        srclist(i)%pn_ext_ml(j) = 0.0   
        srclist(i)%m1_ext_ml(j) = 0.0 
        srclist(i)%m2_ext_ml(j) = 0.0 
        srclist(i)%ep_ext_ml(j) = 0.0
     end do
     srclist(i)%ra=0.0
     srclist(i)%dec = 0.0
     srclist(i)%ra_corr = 0.0
     srclist(i)%dec_corr = 0.0
     srclist(i)%lii =0.0
     srclist(i)%bii =0.0
     srclist(i)%lii_corr=0.0
     srclist(i)%bii_corr=0.0
     srclist(i)%radec_err=0.0
     srclist(i)%pn_tot = -9.999e30  
     srclist(i)%pn_tot_err = -9.999e30
     srclist(i)%m1_tot = -9.999e30
     srclist(i)%m1_tot_err = -9.999e30
     srclist(i)%m2_tot = -9.999e30
     srclist(i)%m2_tot_err = -9.999e30
     srclist(i)%ep_tot = -9.999e30
     srclist(i)%ep_tot_err = -9.999e30
     srclist(i)%pn_xid = -9.999e30
     srclist(i)%pn_xid_err  = -9.999e30
     srclist(i)%m1_xid  = -9.999e30
     srclist(i)%m1_xid_err  = -9.999e30
     srclist(i)%m2_xid = -9.999e30
     srclist(i)%m2_xid_err = -9.999e30
     srclist(i)%ep_xid = -9.999e30
     srclist(i)%ep_xid_err = -9.999e30
     srclist(i)%pn_extent = -9.999e30 
     srclist(i)%pn_ext_err = -9.999e30  
     srclist(i)%m1_extent = -9.999e30 
     srclist(i)%m1_ext_err = -9.999e30
     srclist(i)%m2_extent = -9.999e30  
     srclist(i)%m2_ext_err = -9.999e30  
     srclist(i)%ep_extent = -9.999e30  
     srclist(i)%ep_ext_err = -9.999e30  
     srclist(i)%pn_hr1  = -9.999e30
     srclist(i)%pn_hr1_err = -9.999e30
     srclist(i)%pn_hr2 = -9.999e30 
     srclist(i)%pn_hr2_err = -9.999e30
     srclist(i)%pn_hr3 = -9.999e30
     srclist(i)%pn_hr3_err = -9.999e30
     srclist(i)%m1_hr1 = -9.999e30
     srclist(i)%m1_hr1_err = -9.999e30  
     srclist(i)%m1_hr2 = -9.999e30
     srclist(i)%m1_hr2_err = -9.999e30
     srclist(i)%m1_hr3 = -9.999e30
     srclist(i)%m1_hr3_err = -9.999e30
     srclist(i)%m2_hr1 = -9.999e30
     srclist(i)%m2_hr1_err = -9.999e30 
     srclist(i)%m2_hr2 = -9.999e30
     srclist(i)%m2_hr2_err= -9.999e30
     srclist(i)%m2_hr3  = -9.999e30
     srclist(i)%m2_hr3_err= -9.999e30
     srclist(i)%ep_hr1  = 0.0
     srclist(i)%ep_hr1_err  = 0.0
     srclist(i)%ep_hr2  = 0.0
     srclist(i)%ep_hr2_err= 0.0
     srclist(i)%ep_hr3  = 0.0
     srclist(i)%ep_hr3_err= 0.0
     srclist(i)%var_stat = 0.0 
     srclist(i)%p_var = 0.0
     srclist(i)%n_bins = 0 
     srclist(i)%var_exp_no = null_entry
     do j=1, 5
        srclist(i)%om_id(j) = null_entry
        srclist(i)%om_id_dist(j) = -9.999e30
     end do
     srclist(i)%n_opt_co = 0
     srclist(i)%omflag = 'NO_OM_DATA'
     srclist(i)%spectra = .FALSE.  
     srclist(i)%tseries = .FALSE. 
     srclist(i)%val_flag = '-' 
     srclist(i)%ver_flag = '-'
     srclist(i)%ver_comm = '-'

    end do

  end subroutine init_list

end module srcmatch_io


